/*
 * Exemplo de implementacao do algoritmo RSA em Java,
 * com os numeros primos p e q e mensagem M fornecidos pelo usuario
 * Para p e q dados, solicita e cifra mensagens M, ate ler M em branco
 * Por Marcio H. C. d'Avila <mhavila@inet.com.br>, 2001
 */
import java.math.BigInteger;
import java.security.SecureRandom;
import java.io.*;

public class ReadRsa
{
  private BigInteger n, d, e;

  public ReadRsa(int bitlen)
  {
    // Escolher p e q numeros primos grandes (numero de bits = bitlen/2)
    // BigInteger(b,c,r) = inteiro positivo randomico de b bits provavelmente primo, com
    // chance de erro de 1 em 2^c, tomando r como fonte de bits para testes de primalidade
    BigInteger p, q;
    String s;
    InputStreamReader isr = new InputStreamReader(System.in);
    BufferedReader br = new BufferedReader(isr);
    System.out.print("p = ");
    try {
      s = br.readLine();
      p = new BigInteger(s);
    } catch(IOException e) {
      p = new BigInteger("5");
      System.out.println("p = 5");
    } catch(NumberFormatException e) {
      p = new BigInteger("5");
      System.out.println("p = 5");
    }
    System.out.print("q = ");
    try {
      s = br.readLine();
      q = new BigInteger(s);
    } catch(IOException e) {
      q = new BigInteger("7");
      System.out.print("q = 7");
    } catch(NumberFormatException e) {
      q = new BigInteger("7");
      System.out.print("q = 7");
    }

    // Calcular n = p * q
    n = p.multiply(q);

    // Calcular o quociente euleriano fi(n) = (p - 1)(q - 1)
    BigInteger fi_n = p.subtract(BigInteger.ONE).multiply(q.subtract(BigInteger.ONE));

    // Encontrar e que seja primo relativo a fi(n), ou MDC(e, fi(n)) = 1
    // A estrategia: encontrar um e pequeno, tentando inicialmente o 3
    // e depois seguindo pelos demais inteiros mpares positivos
    e = new BigInteger("3");
    while(fi_n.gcd(e).intValue() > 1) e = e.add(new BigInteger("2"));

    // Calcular d como o inverso multiplicativo de e mod fi(n), ou d*e = 1 mod fi(n)
    d = e.modInverse(fi_n);
  }

  public BigInteger encrypt(BigInteger message)
  {
    return message.modPow(e, n);
  }

  public BigInteger decrypt(BigInteger message)
  {
    return message.modPow(d, n);
  }

  static public void main(String [] args)
  {
    System.out.println("Inicializando RSA...");
    ReadRsa rsa = new ReadRsa(1024);
    System.out.println("");
    System.out.println("n = " + rsa.n);
    System.out.println("e = " + rsa.e);
    System.out.println("d = " + rsa.d);
    System.out.println("");

    System.out.println("Encriptando e Decriptando...");
    BigInteger M;
    InputStreamReader isr = new InputStreamReader(System.in);
    BufferedReader br = new BufferedReader(isr);
    System.out.print("M = ");
    try {
      String s = br.readLine();
      M = new BigInteger(s);
    } catch(IOException e) {
      M = new BigInteger("10");
      System.out.println("M = 10");
    } catch(NumberFormatException e) {
      M = new BigInteger("10");
      System.out.println("M = 10");
    }

    BigInteger C = rsa.encrypt(M);
    System.out.println("C = " + C);
    BigInteger m = rsa.decrypt(C);
    System.out.println("m = " + m);
  }
} // class ReadRsa

